package com.malk.zhixingtongde.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import net.sourceforge.pinyin4j.PinyinHelper;

import com.alibaba.fastjson.JSONObject;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.aliwork.YDClient;
import com.malk.service.dingtalk.DDClient;
import com.malk.utils.UtilEnv;
import com.malk.utils.UtilExcel;
import com.malk.utils.UtilHttp;
import com.malk.utils.UtilMap;
import com.malk.zhixingtongde.entity.Accounts;
import com.malk.zhixingtongde.entity.Cash;
import com.malk.zhixingtongde.entity.Profit;
import com.malk.zhixingtongde.service.ZxtdReportService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import java.io.*;
import java.math.RoundingMode;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ZxtdReportServiceImpl implements ZxtdReportService {

    @Autowired
    private YDClient ydClient;

    @Autowired
    private YDConf ydConf;

    @Autowired
    private DDConf ddConf;

    @Autowired
    private DDClient ddClient;

    private static String[] PROFIT_CLASSIFYS={"收入","销售费用","管理费用","研发费用","财务费用"};// 分类
    private static int[] PROFIT_NUMBERS={15,25,25,25,10};// 报表默认长度

    private static String[] RECEIVABLE_CLASSIFYS ={"平台服务费","技术服务费","其他"};// 应收账款表子科目
    private static int[] RECEIABLE_NUMBERS ={30,30,30};// 应收账款表默认长度

    private static String[] OTHER_RECEIVABLE_CLASSIFYS ={"房屋押金","员工借款（备用金）","往来"};// 其他应收款表子科目
    private static int[] OTHER_RECEIABLE_NUMBERS ={15,15,15};// 其他应收款表默认长度

    private static String[] PAYABLE_CLASSIFYS ={"往来"};// 应付款表子科目
    private static String[] OTHER_PAYABLE_CLASSIFYS ={"往来"};// 其他应付款表子科目
    private static int[] PAYABLE_NUMBERS ={10};// 应付款表默认长度
    private static int[] OTHER_PAYABLE_NUMBERS ={15};// 其他应付款表默认长度

    //实例流程状态完成
    private static Map COMPLETED_STATUS = UtilMap.map("key, value, type, operator, componentName","processInstanceStatus","COMPLETED","ARRAY","in","SelectField");

    //实例流程结果同意
    private static String[] RESULT = {"agree"};
    private static Map AGREE_RESULT = UtilMap.map("key, value, type, operator, componentName","processApprovedResult",RESULT,"ARRAY","in","SelectField");


    String _matchFormUuid(String code) {
        Map<String, String> formUuid = UtilMap.empty();
        if (UtilEnv.getActiveProfile().equals(UtilEnv.ENV_PROD)) {
            formUuid.put("PROFIT", "FORM-32937639BC314882AF45201131EAE4C0YD2L"); // 利润表
            formUuid.put("DOMAIN", "https://nxcyz4.aliwork.com/"); // 宜搭域名
            formUuid.put("BUDGET","FORM-A0B077B5775341869605B2EB7E79356A7ZX7");// 成本费用预算编制中间表
            formUuid.put("INCOME_BUDGET","FORM-3563C85B564345C4B2931CB27AE768C6CE73");// 收入预算编制中间表
            formUuid.put("RECEIVABLE","FORM-F5ADEDFE47C749BF8109BFC4174E70483716");// 应收账款表
            formUuid.put("OTHER_RECEIVABLE","FORM-FAD22D32A4524658B85C8A8482FDDFD0CEW0");// 其他应收账款表
            formUuid.put("PAYABLE","FORM-9594FC6E0BE341DEA83BE378632BBBBD7LEF");// 应付账款表
            formUuid.put("OTHER_PAYABLE","FORM-48CCA085CFDF4DC3B036A880F77842ED273S");// 其他应付账款表
            formUuid.put("INCOME","FORM-DA4BC7EA522E4D59A9233C507206D291N1XX");// 收入申请表
            formUuid.put("PAY","FORM-DE36121157B74774B5C8462E38519269OAVS");// 支出申请表
            formUuid.put("ARTIFICIAL","FORM-52CEBFBA6D56413993E1F5075A493FDCVXVT");// 资产负债手动维护表
        } else {
            formUuid.put("DOMAIN", "https://nxcyz4.aliwork.com/");
        }
        return formUuid.get(code);
    }

    @Override
    public void exportProfit(HttpServletResponse response,String year) {
        Map profit = getProfit(year);
        List<Profit> dataList = (List<Profit>) profit.get("dataList");
        Map mainData = (Map) profit.get("mainData");

        UtilExcel.exportMapAndListByTemplate(response,mainData,dataList, Profit.class, "利润", "Template_profit.xlsx");
    }

    public Map getProfit(String year) {
        // 1.获取宜搭数据
        Profit[] PROFIT_SUM=new Profit[5];// 分类小计数据
        List<Profit> dataList=getData(year,PROFIT_SUM);
        // 计算主数据
        Map mainData=new HashMap();
        mainData.put("year",year);

        // 计算费用合计 与 净利润
        String payTotalKey="payTotalAmt";
        String payTotalAmtTotal="0";
        String netProfitKey="netProfitAmt";
        String netProfitTotal="0";
        for (int i = 1; i < 13; i++) {
            // 费用合计
            String amt=addAmt(PROFIT_SUM,1,4,i);// 第一位数据是收入，不参与计算
            mainData.put(payTotalKey+i,amt);
            payTotalAmtTotal=NumberUtil.add(amt,payTotalAmtTotal).toString();
            // 净利润
            String net=subAmt(PROFIT_SUM,1,4,i);
            mainData.put(netProfitKey+i,net);
            netProfitTotal=NumberUtil.add(netProfitTotal,net).toString();
        }
        mainData.put("payTotalAmtTotal",payTotalAmtTotal);
        mainData.put("netProfitTotal",netProfitTotal);


        //获取股东分红、长期投资、退投资款
        List<Map> payDataList = getYdFormDataList(_matchFormUuid("PAY"),JSONObject.toJSONString(UtilMap.map("textField_lqvpypje", year)),YDConf.FORM_QUERY.retrieve_list);
        String fenhong = "fenhong";
        String touzi = "touzi";
        String tuitouzi = "tuitouzi";
        if (!payDataList.isEmpty()) {
            for (Map payData : payDataList) {
                Map formData = (Map) payData.get("formData");
                String subject = formData.get("selectField_lp9j90w8") == null ? "" : formData.get("selectField_lp9j90w8").toString();
                if (Objects.nonNull(formData.get("textField_lqvpypje"))){
                    String year2 = formData.get("textField_lqvpypje").toString();
                    String month = year2.substring(6);
                    String amt = formData.get("numberField_lp82nads").toString();

                    if(subject.equals("FH")){
                        mainData.put(fenhong+month,NumberUtil.add(mainData.get(fenhong+month) == null ? "0.00" : mainData.get(fenhong+month).toString(),amt).toString());
                        mainData.put(fenhong+"Total",NumberUtil.add(mainData.get(fenhong+"Total") == null ? "0.00" : mainData.get(fenhong+"Total").toString(),amt).toString());
                    }

                    if(subject.equals("对外投资")){
                        mainData.put(touzi+month,NumberUtil.add(mainData.get(touzi+month) == null ? "0.00" : mainData.get(touzi+month).toString(),amt).toString());
                        mainData.put(touzi+"Total",NumberUtil.add(mainData.get(touzi+"Total") == null ? "0.00" : mainData.get(touzi+"Total").toString(),amt).toString());
                    }
                    if(subject.equals("退投资款")){
                        mainData.put(tuitouzi+month,NumberUtil.add(mainData.get(tuitouzi+month) == null ? "0.00" : mainData.get(tuitouzi+month).toString(),amt).toString());
                        mainData.put(tuitouzi+"Total",NumberUtil.add(mainData.get(tuitouzi+"Total") == null ? "0.00" : mainData.get(tuitouzi+"Total").toString(),amt).toString());
                    }
                }
            }
        }

        //获取剩余净利润
        for (int i = 1; i < 13; i++) {
            //剩余净利润=净利润-分红-长期投资-退投资款
            String netProfitAmt = mainData.get(netProfitKey + i) == null ? "0.00" : String.valueOf(mainData.get(netProfitKey + i));
            String fenhongAmt = mainData.get(fenhong + i) == null ? "0.00" : String.valueOf(mainData.get(fenhong + i));
            String touziAmt = mainData.get(touzi + i) == null ? "0.00" : String.valueOf(mainData.get(touzi + i));
            String tuitouziAmt = mainData.get(tuitouzi + i) == null ? "0.00" : String.valueOf(mainData.get(tuitouzi + i));

            String remainNet=NumberUtil.sub(netProfitAmt,fenhongAmt,touziAmt,tuitouziAmt).toString();
            mainData.put("remainNet"+i,remainNet);
        }
        mainData.put("remainNetTotal",NumberUtil.sub(netProfitTotal,
                mainData.get(fenhong+"Total") == null ? "0.00" : String.valueOf(mainData.get(fenhong+"Total")),
                mainData.get(touzi+"Total") == null ? "0.00" : String.valueOf(mainData.get(touzi+"Total")),
                mainData.get(tuitouzi+"Total") == null ? "0.00" : String.valueOf(mainData.get(tuitouzi+"Total")))
                .toString());

        Map map = new HashMap();
        map.put("dataList",dataList);
        map.put("mainData",mainData);

        return map;
    }

    @Override
    public void exportQuarterProfit(HttpServletResponse response, String year) {
        Map quarter = getQuarter(year);
        List<Profit> dataList = (List<Profit>) quarter.get("dataList");
        Map mainData = (Map) quarter.get("mainData");

        UtilExcel.exportMapAndListByTemplate(response,mainData,dataList, Profit.class, "季度利润", "Quarter_Template_profit.xlsx");
    }

    private Map getQuarter(String year) {
        // 1.获取宜搭数据
        Profit[] PROFIT_SUM=new Profit[5];// 分类小计数据
        List<Profit> dataList=getQuarterData(year,PROFIT_SUM);
        // 计算主数据
        Map mainData=new HashMap();
        mainData.put("year",year);
        // 计算费用合计 与 净利润
        String payQuarterTotalKey="payQuarterTotalAmt";
        String payQuarterTotalAmtTotal="0";
        String payBudgetQuarterTotalKey="payBudgetQuarterTotalAmt";
        String payBudgetQuarterTotalAmtTotal="0";
        String netQuarterProfitKey="netQuarterProfitAmt";
        String netQuarterProfitTotal="0";
        String netBudgetQuarterProfitKey="netBudgetQuarterProfitAmt";
        String netBudgetQuarterProfitTotal="0";

        for (int i = 1; i < 5; i++) {
            // 费用合计
            String quarterAmt=addQuarterAmt(PROFIT_SUM,1,4,i);// 第一位数据是收入，不参与计算
            String budgetQuarterAmt=addBudgetQuarterAmt(PROFIT_SUM,1,4,i);// 第一位数据是收入，不参与计算
            mainData.put(payQuarterTotalKey+i,quarterAmt);
            mainData.put(payBudgetQuarterTotalKey+i,budgetQuarterAmt);
            payQuarterTotalAmtTotal=NumberUtil.add(quarterAmt,payQuarterTotalAmtTotal).toString();
            payBudgetQuarterTotalAmtTotal=NumberUtil.add(budgetQuarterAmt,payBudgetQuarterTotalAmtTotal).toString();
            // 净利润
            String net=subQuarterAmt(PROFIT_SUM,1,4,i);
            String budgetNet=subBudgetQuarterAmt(PROFIT_SUM,1,4,i);
            mainData.put(netQuarterProfitKey+i,net);
            mainData.put(netBudgetQuarterProfitKey+i,budgetNet);
            netQuarterProfitTotal=NumberUtil.add(netQuarterProfitTotal,net).toString();
            netBudgetQuarterProfitTotal=NumberUtil.add(netBudgetQuarterProfitTotal,budgetNet).toString();
        }
        mainData.put("payQuarterTotalAmtTotal",payQuarterTotalAmtTotal);
        mainData.put("payBudgetQuarterTotalAmtTotal",payBudgetQuarterTotalAmtTotal);
        mainData.put("netQuarterProfitTotal",netQuarterProfitTotal);
        mainData.put("netBudgetQuarterProfitTotal",netBudgetQuarterProfitTotal);

        Map map = new HashMap();

        map.put("dataList",dataList);
        map.put("mainData",mainData);

        return map;
    }

    @Override
    public void exportReceivableDetails(HttpServletResponse response, String year) {
        Map receivable = getReceivable(year);
        List<Accounts> dataList = (List<Accounts>) receivable.get("dataList");
        Map mainData = (Map) receivable.get("mainData");

        UtilExcel.exportMapAndListByTemplate(response,mainData,dataList, Accounts.class, "应收账款表", "Template_receivable.xlsx");
    }

    private Map getReceivable(String year) {
        // 1.获取宜搭数据
        Accounts[] ACCOUNTS_SUM=new Accounts[3];// 分类小计数据
        List<Accounts> dataList=getReceivableData(year,ACCOUNTS_SUM);
        // 2.计算主数据
        Map mainData=new HashMap();
        mainData.put("year",year);

        Map map = new HashMap();
        map.put("mainData",mainData);
        map.put("dataList",dataList);

        return map;
    }

    @Override
    public void exportOtherReceivableDetails(HttpServletResponse response, String year) {
        Map otherReceivable = getOtherReceivable(year);
        List<Accounts> dataList = (List<Accounts>) otherReceivable.get("dataList");
        Map mainData = (Map) otherReceivable.get("mainData");

        UtilExcel.exportMapAndListByTemplate(response,mainData,dataList, Accounts.class, "其他应收款表", "Template_other_receivable.xlsx");
    }

    private Map getOtherReceivable(String year) {
        // 1.获取宜搭数据
        Accounts[] ACCOUNTS_SUM=new Accounts[3];// 分类小计数据
        List<Accounts> dataList=getOtherReceivableData(year,ACCOUNTS_SUM);
        // 2.计算主数据
        Map mainData=new HashMap();
        mainData.put("year",year);

        Map map = new HashMap();
        map.put("mainData",mainData);
        map.put("dataList",dataList);

        return map;
    }

    @Override
    public void exportOtherPayableDetails(HttpServletResponse response, String year) {
        Map otherPayable = getOtherPayable(year);
        List<Accounts> dataList = (List<Accounts>) otherPayable.get("dataList");
        Map mainData = (Map) otherPayable.get("mainData");

        UtilExcel.exportMapAndListByTemplate(response,mainData,dataList, Accounts.class, "其他应付款表", "Template_other_payable.xlsx");
    }

    private Map getOtherPayable(String year) {
        // 1.获取宜搭数据
        Accounts[] ACCOUNTS_SUM=new Accounts[1];// 分类小计数据
        List<Accounts> dataList=getOtherPayableData(year,ACCOUNTS_SUM);
        // 2.计算主数据
        Map mainData=new HashMap();
        mainData.put("year",year);

        Map map = new HashMap();
        map.put("mainData",mainData);
        map.put("dataList",dataList);

        return map;
    }

    @Override
    public void exportAll(HttpServletResponse response, String year) {
        //利润表
        Map map1 = getProfit(year);

        //资产负债表
        Map map2 = getLiabilities(year);

        //应收账款表
        Map map3 = getReceivable(year);

        //其他应收款表
        Map map4 = getOtherReceivable(year);

        //应付账款表
        Map map5 = getPayable(year);

        //其他应付款表
        Map map6 = getOtherPayable(year);

        //季度核算表
        Map map7 = getQuarter(year);

        List<Object> dataMainList = new ArrayList<>();
        dataMainList.add(map1.get("mainData"));
        dataMainList.add(map2.get("mainData"));
        dataMainList.add(map3.get("mainData"));
        dataMainList.add(map4.get("mainData"));
        dataMainList.add(map5.get("mainData"));
        dataMainList.add(map6.get("mainData"));
        dataMainList.add(map7.get("mainData"));

        List<List> dataLists = new ArrayList<>();
        dataLists.add((List)map1.get("dataList"));
        dataLists.add((List)map2.get("dataList"));
        dataLists.add((List)map3.get("dataList"));
        dataLists.add((List)map4.get("dataList"));
        dataLists.add((List)map5.get("dataList"));
        dataLists.add((List)map6.get("dataList"));
        dataLists.add((List)map7.get("dataList"));

        UtilExcel.exportAllSheetByTemplate(response,dataMainList,dataLists,year,"相关账户明细表","Template_all.xlsx");
    }

    @Override
    public void exportCash(HttpServletResponse response, String year) {
        //现金流量表主表
        Map map1 = getMainCash(year);

        //现金流量表附表
        Map map2 = getSubCash(year);

        List<Object> dataMainList = new ArrayList<>();
        dataMainList.add(map1.get("mainData"));
        dataMainList.add(map2.get("mainData"));

        List<List> dataLists = new ArrayList<>();
        dataLists.add((List)map1.get("dataList"));
        dataLists.add((List)map2.get("dataList"));

        UtilExcel.exportAllSheetByTemplate(response,dataMainList,dataLists,year,"现金流量表","Template_cash.xlsx");
    }

    @Override
    @Async
    public void uploadToDingTalk(HttpServletResponse response,String userId) {
        //获取当前时间戳
        long beginTimestamp = System.currentTimeMillis();
        System.out.println("开始上传文件：" + beginTimestamp);

        //父节点dentryUuid
//        String parentDentryUuid = "9bN7RYPWdMm4Mv4eTDnleGRaVZd1wyK0";//poc api上传测试文件夹
        String parentDentryUuid = "ZX6GRezwJlRgBQmQCGE54DMQWdqbropQ";//知行同德
        String year = DateUtil.year(new Date()) + "";
        String month = DateUtil.month(new Date()) + 1 + "";

        exportAll(response, year);
        exportCash(response, year + "0" + month);

        //文件
        File file1 = new File("d:\\现金流量表-" + year + "0" + month + ".xlsx");
        File file2 = new File("d:\\相关账户明细表-" + year + ".xlsx");
        List<File> files = new ArrayList<>();
        Collections.addAll(files,file1,file2);

        //获取文件上传信息
        Map param = new HashMap();
//        param.put("unionId","5PK3cMYhiStn7HDoNabweEgiEiE");//poc wzy
        param.put("unionId","b80wtLNgk1gXHvKn9eozugiEiE");//知行同德 lfx
        Map body = new HashMap();
        body.put("protocol","HEADER_SIGNATURE");

        for (File file : files) {
            DDR_New ddr = (DDR_New) UtilHttp.doPost("https://api.dingtalk.com/v2.0/storage/spaces/files/" + parentDentryUuid + "/uploadInfos/query", ddConf.initTokenHeader(ddClient.getAccessToken()), param, body, DDR_New.class);
            String uploadKey = ddr.getUploadKey();
            Map headerSignatureInfo = ddr.getHeaderSignatureInfo();
            Map<String,String> headers = (Map<String,String>) headerSignatureInfo.get("headers");
            List<String> resourceUrls = (List<String>) headerSignatureInfo.get("resourceUrls");
            String resourceUrl = resourceUrls.get(0);

            //使用OSS的header加签方式上传文件
            try {
                URL url = new URL(resourceUrl);
                HttpURLConnection connection = (HttpURLConnection)url.openConnection();
                if (headers != null) {
                    for (Map.Entry<String, String> entry : headers.entrySet()) {
                        connection.setRequestProperty(entry.getKey(), entry.getValue());
                    }
                }
                connection.setDoOutput(true);
                connection.setRequestMethod("PUT");
                connection.setUseCaches(false);
                connection.setReadTimeout(10000);
                connection.setConnectTimeout(10000);
                connection.connect();
                OutputStream out = connection.getOutputStream();


                    InputStream is = new FileInputStream(file);
                    byte[] b =new byte[1024];
                    int temp;
                    while ((temp=is.read(b))!=-1){
                        out.write(b,0,temp);
                    }
                    out.flush();
                    out.close();
                    int responseCode = connection.getResponseCode();
                    connection.disconnect();
                    if (responseCode == 200) {
                        System.out.println("上传成功");
                    } else {
                        System.out.println("上传失败");
                    }

                    //提交文件
                    Map body2 = new HashMap();
                    Map option = new HashMap();
                    option.put("conflictStrategy","OVERWRITE");
                    body2.put("uploadKey",uploadKey);
                    body2.put("name",file.getName());
                    body2.put("option",option);

                    UtilHttp.doPost("https://api.dingtalk.com/v2.0/storage/spaces/files/" + parentDentryUuid + "/commit",ddClient.initTokenHeader(),param,body2);

            }catch (IOException e){
                System.out.println("上传文件异常");
            }
        }
        long endTimestamp = System.currentTimeMillis();
        System.out.println("上传文件结束：" + endTimestamp);
        System.out.println("上传文件耗时：" + (endTimestamp - beginTimestamp)/1000 + "s");

        //发送工作通知给当前登陆人
        if (Objects.nonNull(userId)){
            Map param2 = new HashMap();
            param2.put("access_token",ddClient.getAccessToken());
            Map body3 = new HashMap();
            body3.put("agent_id",ddConf.getAgentId());
            body3.put("userid_list",userId);
            Map msg = new HashMap();
            msg.put("msgtype","text");
            //获取当前时间 精确到秒
            String time = DateUtil.format(new Date(),"yyyy-MM-dd HH:mm:ss");
            msg.put("text",UtilMap.map("content","报表已同步完成！请到钉钉云盘查看  时间：" + time));
            body3.put("msg",msg);
            UtilHttp.doPost("https://oapi.dingtalk.com/topapi/message/corpconversation/asyncsend_v2",null,param2,body3);
        }
    }

    @Override
    public void exportPayableDetails(HttpServletResponse response, String year) {
        Map payable = getPayable(year);
        List<Accounts> dataList = (List<Accounts>) payable.get("dataList");
        Map mainData = (Map) payable.get("mainData");

        UtilExcel.exportMapAndListByTemplate(response,mainData,dataList, Accounts.class, "应付账款表", "Template_payable.xlsx");
    }

    @Override
    public void exportLiabilitiesDetails(HttpServletResponse response, String year) {
        Map liabilities = getLiabilities(year);
        List<Accounts> dataList = (List<Accounts>) liabilities.get("dataList");
        Map mainData = (Map) liabilities.get("mainData");

        UtilExcel.exportMapAndListByTemplate(response,mainData,dataList, Accounts.class, "应付账款表", "Template_liabilities.xlsx");
    }

    private Map getLiabilities(String year) {
        // 1.获取宜搭数据
        Accounts[] ACCOUNTS_SUM=new Accounts[17];// 项目内容
        List<Accounts> dataList=getLiabilitiesData(year,ACCOUNTS_SUM);
        // 2.计算主数据
        Map mainData=new HashMap();
        mainData.put("year",year);

        Map map = new HashMap();
        map.put("mainData",mainData);
        map.put("dataList",dataList);

        return map;
    }

    private List<Accounts> getLiabilitiesData(String year, Accounts[] ACCOUNTS_SUM) {
        List<Accounts> dataList = new ArrayList<>();

        String dateParam = getYearParam(year);

        //今年之前所有收入
        List<Map> lastIncomeJsonString = new ArrayList<>();
        lastIncomeJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvp25z0",year,"TEXT","lt","TextField"));
        lastIncomeJsonString.add(COMPLETED_STATUS);
        lastIncomeJsonString.add(AGREE_RESULT);
        String lastIncomeSearchCondition = JSONObject.toJSONString(lastIncomeJsonString);
        List<Map> lastIncomeList = getYdFormDataList(_matchFormUuid("INCOME"),lastIncomeSearchCondition,YDConf.FORM_QUERY.retrieve_list);
        //今年之前所有支出
        List<Map> lastPayJsonString = new ArrayList<>();
        lastPayJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvpypje",year,"TEXT","lt","TextField"));
        lastPayJsonString.add(COMPLETED_STATUS);
        lastPayJsonString.add(AGREE_RESULT);
        String lastPaySearchCondition = JSONObject.toJSONString(lastPayJsonString);
        List<Map> lastPayList = getYdFormDataList(_matchFormUuid("PAY"),lastPaySearchCondition,YDConf.FORM_QUERY.retrieve_list);


        String initialAmt1 = "3061971.92";

        if (!lastIncomeList.isEmpty()){
            for (Map lastIncome : lastIncomeList) {
                Map formData = (Map) lastIncome.get("formData");
                if (Objects.nonNull(formData.get("textField_lqvp25z0"))){
                    initialAmt1 = NumberUtil.add(initialAmt1,getBigDecimalStringAmt(formData.get("numberField_lp8278sd"))).toString();
                }
            }
        }

        if (!lastPayList.isEmpty()){
            for (Map lastPay : lastPayList) {
                Map formData = (Map) lastPay.get("formData");
                if (Objects.nonNull(formData.get("textField_lqvpypje"))){
                    initialAmt1 = NumberUtil.add(initialAmt1,getBigDecimalStringAmt(formData.get("numberField_lp82nads"))).toString();
                }
                if (Objects.nonNull(formData.get("multiSelectField_lv0bm8qs"))){
                    List<String> accounts = (List<String>) formData.get("multiSelectField_lv0bm8qs");
                    String account = "";
                    if (Objects.nonNull(accounts) && !accounts.isEmpty()){
                        account = accounts.get(0);
                    }
                }
            }
        }

        //查询收入明细表
        List<Map> incomeJsonString = new ArrayList<>();
        incomeJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvp25z0",year,"TEXT","like","TextField"));
        incomeJsonString.add(COMPLETED_STATUS);
        incomeJsonString.add(AGREE_RESULT);
        String incomeSearchCondition = JSONObject.toJSONString(incomeJsonString);
        List<Map> incomeList = getYdFormDataList(_matchFormUuid("INCOME"),incomeSearchCondition,YDConf.FORM_QUERY.retrieve_list);

        //查询支出明细表
        List<Map> payJsonString = new ArrayList<>();
        payJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvpypje",year,"TEXT","like","TextField"));
        payJsonString.add(COMPLETED_STATUS);
        payJsonString.add(AGREE_RESULT);
        String paySearchCondition = JSONObject.toJSONString(payJsonString);
        List<Map> payList = getYdFormDataList(_matchFormUuid("PAY"),paySearchCondition,YDConf.FORM_QUERY.retrieve_list);

        // 查询应收账款表数据
        List<Map> receivableList = getYdFormDataList(_matchFormUuid("RECEIVABLE"),JSONObject.toJSONString(UtilMap.map("dateField_lqyx5k0n", dateParam)),YDConf.FORM_QUERY.retrieve_list_all);

        // 查询其他应收款表
        List<Map> otherReceivableList = getYdFormDataList(_matchFormUuid("OTHER_RECEIVABLE"),JSONObject.toJSONString(UtilMap.map("dateField_lqyx5k0n",dateParam)),YDConf.FORM_QUERY.retrieve_list_all);

        // 查询应付账款表
        List<Map> payableList = getYdFormDataList(_matchFormUuid("PAYABLE"),JSONObject.toJSONString(UtilMap.map("dateField_lqyx5k0n",dateParam)),YDConf.FORM_QUERY.retrieve_list_all);

        // 查询其他应付款表
        List<Map> otherPayableList = getYdFormDataList(_matchFormUuid("OTHER_PAYABLE"),JSONObject.toJSONString(UtilMap.map("dateField_lqyx5k0n",dateParam)),YDConf.FORM_QUERY.retrieve_list_all);

        // 查询利润统计表
        List<Map> profitList = getYdFormDataList(_matchFormUuid("PROFIT"),JSONObject.toJSONString(UtilMap.map("textField_lqvp89br",year)),YDConf.FORM_QUERY.retrieve_list);

        //查询资产负债手动维护表
        List<Map> artificialList = getYdFormDataList(_matchFormUuid("ARTIFICIAL"),JSONObject.toJSONString(UtilMap.map("textField_lxtzn9up",year)),YDConf.FORM_QUERY.retrieve_list_all);


        //货币资金
        Accounts accounts1 = new Accounts();

        accounts1.setInitialAmt(getBigDecimalStringAmt(initialAmt1));

        if (!incomeList.isEmpty()){
            for (Map income : incomeList) {
                Map formData = (Map) income.get("formData");
                if (Objects.nonNull(formData.get("textField_lqvp25z0")) && Objects.nonNull(formData.get("selectField_lqga7tw0"))){
                    String month = formData.get("textField_lqvp25z0").toString().substring(5);
                    String amt = getBigDecimalStringAmt(formData.get("numberField_lp8278sd"));
                    ReflectUtil.invoke(accounts1, "setAmt"+month, NumberUtil.add(amt,ReflectUtil.invoke(accounts1,"getAmt"+month)).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                }
            }
        }
        if (!payList.isEmpty()){
            for (Map pay : payList) {
                Map formData = (Map) pay.get("formData");
                if (Objects.nonNull(formData.get("textField_lqvpypje")) && Objects.nonNull(formData.get("multiSelectField_lv0bm8qs"))){
                    String month = formData.get("textField_lqvpypje").toString().substring(6);
                    String amt = getBigDecimalStringAmt(formData.get("numberField_lp82nads"));

                    ReflectUtil.invoke(accounts1, "setAmt"+month, NumberUtil.sub(ReflectUtil.invoke(accounts1,"getAmt"+month),amt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                    List<String> accounts = (List<String>) formData.get("multiSelectField_lv0bm8qs");
                    String account = "";
                    if (Objects.nonNull(accounts) && !accounts.isEmpty()){
                        account = accounts.get(0);
                    }
                }
            }
        }


        //月末时货币资金金额
        String monthAmt1 = initialAmt1;

        //获取当前月份
        int month1 = DateUtil.month(new Date());

        for (int i = 1; i < month1 + 2; i++) {
            monthAmt1 = NumberUtil.add(monthAmt1,ReflectUtil.invoke(accounts1,"getAmt"+i)).setScale(2,BigDecimal.ROUND_HALF_UP).toString();
            ReflectUtil.invoke(accounts1, "setAmt"+i, monthAmt1);
       }


        //应收账款
        Accounts accounts3 = new Accounts();

        if (!receivableList.isEmpty()){
            for (Map receivable : receivableList) {
                Map formData = (Map) receivable.get("formData");
                Accounts accounts = new Accounts();
                List<Map> receivableDetails = (List<Map>) formData.get("tableField_lqyx5k0c");
                if (Objects.nonNull(receivableDetails) && !receivableDetails.isEmpty()){
                    for (Map receivableDetail : receivableDetails) {
                        accounts.setInitialAmt(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0o")));
                        accounts.setAmt1(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0q")));
                        accounts.setAmt2(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0r")));
                        accounts.setAmt3(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0s")));
                        accounts.setAmt4(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0t")));
                        accounts.setAmt5(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0u")));
                        accounts.setAmt6(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0v")));
                        accounts.setAmt7(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0w")));
                        accounts.setAmt8(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0x")));
                        accounts.setAmt9(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0y")));
                        accounts.setAmt10(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k0z")));
                        accounts.setAmt11(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k10")));
                        accounts.setAmt12(getBigDecimalStringAmt(receivableDetail.get("numberField_lqyx5k11")));

                        accounts3.add(accounts);
                    }
                }
            }
            //获取收入中实际已收的金额-应收账款表
            List<Map> collect = incomeList.stream().filter(p ->
                            String.valueOf(((Map) p.get("formData")).get("selectField_ly2pd5cm")).equals("关联应收帐款表"))
                    .collect(Collectors.toList());
            //应收-已收
            if (!collect.isEmpty()){
                for (Map oneCollect : collect) {
                    Map formData1 = (Map) oneCollect.get("formData");
                    String amt = String.valueOf(formData1.get("numberField_lp8278sd"));
                    String month = String.valueOf(formData1.get("textField_lxl8moor")).substring(6);
                    ReflectUtil.invoke(accounts3,"setAmt"+month,NumberUtil.sub(ReflectUtil.invoke(accounts3,"getAmt"+month),amt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                }
            }
            String addAmt = accounts3.getInitialAmt();
            for (int i = 1; i < month1+2; i++) {
                ReflectUtil.invoke(accounts3,"setAmt"+i,NumberUtil.add(ReflectUtil.invoke(accounts3,"getAmt"+i),addAmt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                addAmt = ReflectUtil.invoke(accounts3,"getAmt"+i);
            }
        }

        //其他应收款
        Accounts accounts2 = new Accounts();

        if (!otherReceivableList.isEmpty()){
            for (Map otherReceivable : otherReceivableList) {
                Map formData = (Map) otherReceivable.get("formData");
                Accounts accounts = new Accounts();
                List<Map> otherReceivableDetails = (List<Map>) formData.get("tableField_lqyx5k0c");
                if (Objects.nonNull(otherReceivableDetails) && !otherReceivableDetails.isEmpty()){
                    for (Map otherReceivableDetail : otherReceivableDetails) {
                        accounts.setInitialAmt(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0o")));
                        accounts.setAmt1(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0q")));
                        accounts.setAmt2(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0r")));
                        accounts.setAmt3(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0s")));
                        accounts.setAmt4(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0t")));
                        accounts.setAmt5(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0u")));
                        accounts.setAmt6(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0v")));
                        accounts.setAmt7(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0w")));
                        accounts.setAmt8(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0x")));
                        accounts.setAmt9(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0y")));
                        accounts.setAmt10(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k0z")));
                        accounts.setAmt11(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k10")));
                        accounts.setAmt12(getBigDecimalStringAmt(otherReceivableDetail.get("numberField_lqyx5k11")));

                        accounts2.add(accounts);
                    }
                }
            }
            //获取收入中实际已收的金额-应收账款表
            List<Map> collect = incomeList.stream().filter(p ->
                            String.valueOf(((Map) p.get("formData")).get("selectField_ly2pd5cm")).equals("关联其他应收款表"))
                    .collect(Collectors.toList());
            //应收-已收
            if (!collect.isEmpty()){
                for (Map oneCollect : collect) {
                    Map formData1 = (Map) oneCollect.get("formData");
                    String amt = String.valueOf(formData1.get("numberField_lp8278sd"));
                    String month = String.valueOf(formData1.get("textField_lxl8moor")).substring(6);
                    ReflectUtil.invoke(accounts2,"setAmt"+month,NumberUtil.sub(ReflectUtil.invoke(accounts2,"getAmt"+month),amt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                }
            }

            String addAmt = accounts2.getInitialAmt();
            for (int i = 1; i < month1+2; i++) {
                ReflectUtil.invoke(accounts2,"setAmt"+i,NumberUtil.add(ReflectUtil.invoke(accounts2,"getAmt"+i),addAmt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                addAmt = ReflectUtil.invoke(accounts2,"getAmt"+i);
            }
        }

        //手填数据
        Accounts accounts4 = new Accounts();
        Accounts accounts5 = new Accounts();
        Accounts accounts6 = new Accounts();
        Accounts accounts8 = new Accounts();
        Accounts accounts9 = new Accounts();
        Accounts accounts14 = new Accounts();
        if (!artificialList.isEmpty()){
            for (Map artificial : artificialList) {
                Map formData = (Map) artificial.get("formData");
                String subject = String.valueOf(formData.get("selectField_lxtqso85"));
                Accounts accounts = new Accounts();
                List<Map> artificialDetails = (List<Map>) formData.get("tableField_lqyx5k0c");
                if (Objects.nonNull(artificialDetails) && !artificialDetails.isEmpty()){
                    for (Map artificialDetail : artificialDetails) {
                        accounts.setInitialAmt(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0o")));
                        accounts.setAmt1(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0q")));
                        accounts.setAmt2(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0r")));
                        accounts.setAmt3(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0s")));
                        accounts.setAmt4(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0t")));
                        accounts.setAmt5(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0u")));
                        accounts.setAmt6(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0v")));
                        accounts.setAmt7(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0w")));
                        accounts.setAmt8(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0x")));
                        accounts.setAmt9(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0y")));
                        accounts.setAmt10(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k0z")));
                        accounts.setAmt11(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k10")));
                        accounts.setAmt12(getBigDecimalStringAmt(artificialDetail.get("numberField_lqyx5k11")));
                        switch (subject){
                            case "固定资产账面价值" : accounts4.add(accounts);break;
                            case "无形资产" : accounts5.add(accounts);break;
                            case "长期投资" : accounts6.add(accounts);break;
                            case "应付职工薪酬" : accounts8.add(accounts);break;
                            case "应交税费" : accounts9.add(accounts);break;
                            case "实收资本" : accounts14.add(accounts);break;
                        }
                    }
                }
            }
        }


        //应付账款
        Accounts accounts12 = new Accounts();

        if (!payableList.isEmpty()){
            for (Map payable : payableList) {
                Map formData = (Map) payable.get("formData");
                Accounts accounts = new Accounts();
                List<Map> payableDetails = (List<Map>) formData.get("tableField_lqyx5k0c");
                if (Objects.nonNull(payableDetails) && !payableDetails.isEmpty()){
                    for (Map payableDetail : payableDetails) {
                        accounts.setInitialAmt(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0o")));
                        accounts.setAmt1(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0q")));
                        accounts.setAmt2(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0r")));
                        accounts.setAmt3(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0s")));
                        accounts.setAmt4(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0t")));
                        accounts.setAmt5(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0u")));
                        accounts.setAmt6(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0v")));
                        accounts.setAmt7(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0w")));
                        accounts.setAmt8(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0x")));
                        accounts.setAmt9(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0y")));
                        accounts.setAmt10(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k0z")));
                        accounts.setAmt11(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k10")));
                        accounts.setAmt12(getBigDecimalStringAmt(payableDetail.get("numberField_lqyx5k11")));

                        accounts12.add(accounts);
                    }
                }
            }
            //获取支出中实际已付的金额-应付账款表
            List<Map> collect = payList.stream().filter(p ->
                            String.valueOf(((Map) p.get("formData")).get("selectField_ly2lhkpe")).equals("关联应付账款表"))
                    .collect(Collectors.toList());
            //应付-已付
            if (!collect.isEmpty()){
                for (Map oneCollect : collect) {
                    Map formData1 = (Map) oneCollect.get("formData");
                    String amt = String.valueOf(formData1.get("numberField_lp82nads"));
                    String month = String.valueOf(formData1.get("textField_lqvpypje")).substring(6);
                    ReflectUtil.invoke(accounts12,"setAmt"+month,NumberUtil.sub(ReflectUtil.invoke(accounts12,"getAmt"+month),amt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                }
            }
            String addAmt = accounts12.getInitialAmt();
            for (int i = 1; i < month1+2; i++) {
                ReflectUtil.invoke(accounts12,"setAmt"+i,NumberUtil.add(ReflectUtil.invoke(accounts12,"getAmt"+i),addAmt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                addAmt = ReflectUtil.invoke(accounts12,"getAmt"+i);
            }

        }



        //其他应付款
        Accounts accounts10 = new Accounts();

        if (!otherPayableList.isEmpty()){
            for (Map otherPayable : otherPayableList) {
                Map formData = (Map) otherPayable.get("formData");
                Accounts accounts = new Accounts();
                List<Map> otherPayableDetails = (List<Map>) formData.get("tableField_lqyx5k0c");
                if (Objects.nonNull(otherPayableDetails) && !otherPayableDetails.isEmpty()){
                    for (Map otherPayableDetail : otherPayableDetails) {
                        accounts.setInitialAmt(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0o")));
                        accounts.setAmt1(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0q")));
                        accounts.setAmt2(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0r")));
                        accounts.setAmt3(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0s")));
                        accounts.setAmt4(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0t")));
                        accounts.setAmt5(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0u")));
                        accounts.setAmt6(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0v")));
                        accounts.setAmt7(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0w")));
                        accounts.setAmt8(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0x")));
                        accounts.setAmt9(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0y")));
                        accounts.setAmt10(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k0z")));
                        accounts.setAmt11(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k10")));
                        accounts.setAmt12(getBigDecimalStringAmt(otherPayableDetail.get("numberField_lqyx5k11")));

                        accounts10.add(accounts);
                    }
                }
            }
            //获取支出中实际已付的金额-其他应付款表
            List<Map> collect = payList.stream().filter(p ->
                            String.valueOf(((Map) p.get("formData")).get("selectField_ly2lhkpe")).equals("关联其他应付款表"))
                    .collect(Collectors.toList());
            //应付-已付
            if (!collect.isEmpty()){
                for (Map oneCollect : collect) {
                    Map formData1 = (Map) oneCollect.get("formData");
                    String amt = String.valueOf(formData1.get("numberField_lp82nads"));
                    String month = String.valueOf(formData1.get("textField_lqvpypje")).substring(6);
                    ReflectUtil.invoke(accounts10,"setAmt"+month,NumberUtil.sub(ReflectUtil.invoke(accounts10,"getAmt"+month),amt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                }
            }

            String addAmt = accounts10.getInitialAmt();
            for (int i = 1; i < month1+2; i++) {
                ReflectUtil.invoke(accounts10,"setAmt"+i,NumberUtil.add(ReflectUtil.invoke(accounts10,"getAmt"+i),addAmt).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
                addAmt = ReflectUtil.invoke(accounts10,"getAmt"+i);
            }
        }

        //未分配利润
        Accounts accounts15 = new Accounts();
        for (Map profit : profitList) {
            Map formData = (Map) profit.get("formData");
            if (Objects.nonNull(formData.get("textField_lqvp89br"))){
                String month = formData.get("textField_lqvp89br").toString().substring(6);
                String amt = getBigDecimalStringAmt(formData.get("numberField_lqqhf8yn"));
                ReflectUtil.invoke(accounts15, "setAmt"+month, NumberUtil.add(amt,ReflectUtil.invoke(accounts15,"getAmt"+month)).setScale(2,BigDecimal.ROUND_HALF_UP).toString());
            }
        }

        for (int i = 1; i <= 15; i++) {
            switch (i){
                case 1:dataList.add(accounts1);break;
                case 2:dataList.add(accounts2);break;
                case 3:dataList.add(accounts3);break;
                case 4:dataList.add(accounts4);break;
                case 5:dataList.add(accounts5);break;
                case 6:dataList.add(accounts6);break;
                case 8:dataList.add(accounts8);break;
                case 9:dataList.add(accounts9);break;
                case 14:dataList.add(accounts14);break;
                case 10:dataList.add(accounts10);break;
                case 12:dataList.add(accounts12);break;
                case 15:dataList.add(accounts15);break;
                default:dataList.add(new Accounts());break;
            }
        }
        return dataList;
    }

    private List<Map> getYdFormDataList(String formUuid, String searchCondition, YDConf.FORM_QUERY formQuery) {
        List<Map> list = new ArrayList<>();
        DDR_New ddrNew = new DDR_New();
        int pageNumber = 1;
        int pageSize = 100;
        do {
            ddrNew = ydClient.queryData(YDParam.builder().formUuid(formUuid)
                    .searchCondition(searchCondition)
                    .pageNumber(pageNumber)
                    .pageSize(pageSize).build(), formQuery);

            list.addAll((List<Map>) ddrNew.getData());
            pageNumber++;
        }while (ddrNew.getTotalCount() > ddrNew.getPageNumber() * pageSize);
        return list;
    }

    //转化为数字字符串 保留两位小数
    private static String getBigDecimalStringAmt(Object object) {
        return object == null ? "0.00" : new BigDecimal(object.toString()).setScale(2, RoundingMode.HALF_UP).toString();
    }

    private Map getPayable(String year) {
        // 1.获取宜搭数据
        Accounts[] ACCOUNTS_SUM=new Accounts[3];// 分类小计数据
        List<Accounts> dataList=getPayableData(year,ACCOUNTS_SUM);
        // 2.计算主数据
        Map mainData=new HashMap();
        mainData.put("year",year);

        Map map = new HashMap();
        map.put("mainData",mainData);
        map.put("dataList",dataList);

        return map;
    }

    private List<Accounts> getPayableData(String year, Accounts[] ACCOUNTS_SUM) {
        String dateParam = getYearParam(year);
        // 查询宜搭底表数据
        List<Map> list = getYdFormDataList(_matchFormUuid("PAYABLE"),JSONObject.toJSONString(UtilMap.map("dateField_lqyx5k0n", dateParam)),YDConf.FORM_QUERY.retrieve_list_all);

        Map<String,Map<String,Accounts>> dataMap=new HashMap<>();
        // 数据预处理
        for (Map map:list){
            JSONObject formData=JSONObject.parseObject(String.valueOf(map.get("formData")));
            String subject = formData.getString("textField_lqyx5k16") ;// 科目

            if (ArrayUtil.contains(PAYABLE_CLASSIFYS,subject)){
                //获取其他应收款明细子表单数据
                List<Map> detailList = (List<Map>) formData.get("tableField_lqyx5k0c");
                Map<String,Accounts> accountsMap = new HashMap<>();
                for (Map detail : detailList) {
                    Accounts accounts = new Accounts();
                    accounts.setCustomerName(detail.get("textField_lqyx5k0f").toString().isEmpty() ? "--" : detail.get("textField_lqyx5k0f").toString());// 客户名称
                    accounts.setInitialAmt(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0o")));// 期初余额
                    accounts.setAmt1(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0q")));// 1月金额
                    accounts.setAmt2(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0r"))); // 2月金额
                    accounts.setAmt3(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0s"))); // 3月金额
                    accounts.setAmt4(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0t"))); // 4月金额
                    accounts.setAmt5(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0u"))); // 5月金额
                    accounts.setAmt6(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0v"))); // 6月金额
                    accounts.setAmt7(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0w"))); // 7月金额
                    accounts.setAmt8(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0x"))); // 8月金额
                    accounts.setAmt9(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0y"))); // 9月金额
                    accounts.setAmt10(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0z"))); // 10月金额
                    accounts.setAmt11(getBigDecimalStringAmt(detail.get("numberField_lqyx5k10"))); // 11月金额
                    accounts.setAmt12(getBigDecimalStringAmt(detail.get("numberField_lqyx5k11"))); // 12月金额
                    //获取原始客户应收账款明细
                    Accounts oldAccounts = accountsMap.get(accounts.getCustomerName());
                    if (Objects.nonNull(oldAccounts)){
                        accounts.add(oldAccounts);
                    }
                    accountsMap.put(accounts.getCustomerName(),accounts);
                }
                Map<String, Accounts> oldAccountsMap = dataMap.get(subject);
                if (Objects.nonNull(oldAccountsMap)){
                    //找出accountsMap和oldAccountsMap相同的key
                    Set<String> customerSet = accountsMap.keySet();
                    for (String oldCustomer : oldAccountsMap.keySet()) {
                        if (customerSet.contains(oldCustomer)){
                            accountsMap.put(oldCustomer,oldAccountsMap.get(oldCustomer).add(accountsMap.get(oldCustomer)));
                        }else {
                            accountsMap.put(oldCustomer,oldAccountsMap.get(oldCustomer));
                        }
                    }
                }
                dataMap.put(subject,accountsMap);
            }
        }

        List<Accounts> accountsList=new ArrayList<>();
        Accounts nullAccounts=new Accounts();
        Accounts totalAccounts = new Accounts();
        // 根据 子科目 处理数据
        for (int i = 0; i < PAYABLE_CLASSIFYS.length; i++) {
            List<Accounts> list1=new ArrayList<>();
            Accounts subTotalAccounts=new Accounts();
            if(dataMap.containsKey(PAYABLE_CLASSIFYS[i])){
                for (String str:dataMap.get(PAYABLE_CLASSIFYS[i]).keySet()){
                    Accounts accountsTemp = dataMap.get(PAYABLE_CLASSIFYS[i]).get(str);
                    list1.add(accountsTemp);
                    subTotalAccounts.setInitialAmt(NumberUtil.add(subTotalAccounts.getInitialAmt(),accountsTemp.getInitialAmt()).toString());
                    for (int j = 1; j < 13; j++) {
                        String amtTemp=ReflectUtil.invoke(accountsTemp,"getAmt"+j);
                        String amt=ReflectUtil.invoke(subTotalAccounts,"getAmt"+j);
                        ReflectUtil.invoke(subTotalAccounts, "setAmt"+j, NumberUtil.add(amt,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                    }
                }
            }
            // 补充空数据
            for (int j = list1.size(); j < PAYABLE_NUMBERS[i]; j++) {
                list1.add(nullAccounts);
            }
            // 添加小计
            ACCOUNTS_SUM[i]=subTotalAccounts;
            list1.add(ACCOUNTS_SUM[i]);
            totalAccounts.add(subTotalAccounts);
            if (i == PAYABLE_NUMBERS.length - 1){
                //添加合计
                list1.add(totalAccounts);
            }
            accountsList.addAll(list1);
        }
        return accountsList;
    }

    private Map getMainCash(String year) {
        //本期收入
        List<Map> incomeJsonString = new ArrayList<>();
        incomeJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvp25z0",year,"TEXT","like","TextField"));
        incomeJsonString.add(COMPLETED_STATUS);
        incomeJsonString.add(AGREE_RESULT);
        String incomeSearchCondition = JSONObject.toJSONString(incomeJsonString);
        List<Map> incomeDataList = getYdFormDataList(_matchFormUuid("INCOME"),incomeSearchCondition,YDConf.FORM_QUERY.retrieve_list);

//        List<Map> incomeDataList = getYdFormDataList(_matchFormUuid("INCOME"),JSONObject.toJSONString(UtilMap.map("textField_lqvp25z0", year)),YDConf.FORM_QUERY.retrieve_list);

        Cash cash = new Cash();

        if (!incomeDataList.isEmpty()){
            for (Map data : incomeDataList) {
                Map formData = (Map) data.get("formData");
                String subject = formData.get("selectField_lpaxbyps") == null ? "" : formData.get("selectField_lpaxbyps").toString();
                //销售产成品、商品、提供劳务收到的现金
                if (ArrayUtil.contains(new String[]{"客户服务费","平台服务费","技术服务费","合规收入","往来款"}, subject)){
                    cash.setAmt1(NumberUtil.add(cash.getAmt1(),formData.get("numberField_lp8278sd").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //收到的税费返还
                if (ArrayUtil.contains(new String[]{"财政返税"}, subject)){
                    cash.setAmt2(NumberUtil.add(cash.getAmt2(),formData.get("numberField_lp8278sd").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //收到其他与经营活动有关的现金
                if (ArrayUtil.contains(new String[]{"代付工资款"}, subject)){
                    cash.setAmt3(NumberUtil.add(cash.getAmt3(),formData.get("numberField_lp8278sd").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //吸收投资收到的现金
                if (ArrayUtil.contains(new String[]{"投资款"}, subject)){
                    cash.setAmt18(NumberUtil.add(cash.getAmt18(),formData.get("numberField_lp8278sd").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
            }
        }

        //上期收入
        String lastYear = (Integer.parseInt(year.substring(0, 4)) - 1) + year.substring(4);


        List<Map> lastIncomeJsonString = new ArrayList<>();
        lastIncomeJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvp25z0",lastYear,"TEXT","like","TextField"));
        lastIncomeJsonString.add(COMPLETED_STATUS);
        lastIncomeJsonString.add(AGREE_RESULT);
        String lastIncomeSearchCondition = JSONObject.toJSONString(lastIncomeJsonString);
        List<Map> lastIncomeDataList = getYdFormDataList(_matchFormUuid("INCOME"),lastIncomeSearchCondition,YDConf.FORM_QUERY.retrieve_list);

//        List<Map> lastIncomeDataList = getYdFormDataList(_matchFormUuid("INCOME"),JSONObject.toJSONString(UtilMap.map("textField_lqvp25z0", lastYear)),YDConf.FORM_QUERY.retrieve_list);

        if (!lastIncomeDataList.isEmpty()){
            for (Map lastData : lastIncomeDataList) {
                Map formData = (Map) lastData.get("formData");
                String subject = formData.get("selectField_lpaxbyps") == null ? "" : formData.get("selectField_lpaxbyps").toString();
                //销售产成品、商品、提供劳务收到的现金
                if (ArrayUtil.contains(new String[]{"客户服务费","平台服务费","技术服务费","合规收入","往来款"}, subject)){
                    cash.setLastAmt1(NumberUtil.add(cash.getLastAmt1(),formData.get("numberField_lp8278sd").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //收到的税费返还
                if (ArrayUtil.contains(new String[]{"财务返税"}, subject)){
                    cash.setLastAmt2(NumberUtil.add(cash.getLastAmt2(),formData.get("numberField_lp8278sd").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //收到其他与经营活动有关的现金
                if (ArrayUtil.contains(new String[]{"存款利息","代付工资款"}, subject)){
                    cash.setLastAmt3(NumberUtil.add(cash.getLastAmt3(),formData.get("numberField_lp8278sd").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //吸收投资收到的现金
                if (ArrayUtil.contains(new String[]{"投资款"}, subject)){
                    cash.setLastAmt18(NumberUtil.add(cash.getLastAmt18(),formData.get("numberField_lp8278sd").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
            }
        }

        //本期支出
        List<Map> payDataList = getYdFormDataList(_matchFormUuid("PAY"),JSONObject.toJSONString(UtilMap.map("textField_lqvpypje", year.substring(0,4) + "-" + year.substring(4))),YDConf.FORM_QUERY.retrieve_list);

        if (!payDataList.isEmpty()) {
            for (Map payData : payDataList) {
                Map formData = (Map) payData.get("formData");
                String subject = formData.get("selectField_lp9j90w8") == null ? "" : formData.get("selectField_lp9j90w8").toString();
                //支付给职工以及为职工支付的现金
                if (ArrayUtil.contains(new String[]{"薪酬及附件","社保及公积金"}, subject)) {
                    cash.setAmt5(NumberUtil.add(cash.getAmt5(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //支付的各项税费
                if (ArrayUtil.contains(new String[]{"经营税费"}, subject)) {
                    cash.setAmt6(NumberUtil.add(cash.getAmt6(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //支付其他与经营活动有关的现金
                if (ArrayUtil.contains(new String[]{"福利费","交通费","差旅费","业务招待费","办公杂费","会议/培训费","快递费","通讯费/网费","广告宣传费","推广佣金","软件/系统服务费","办公设备/软件","物管水电","办公房屋费用","公车费用","代办手续费/税费","客户退款","外包服务费","押金","往来款","手续费"}, subject)){
                    cash.setAmt7(NumberUtil.add(cash.getAmt7(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }

                //投资支付的现金
                if (ArrayUtil.contains(new String[]{"对外投资款"}, subject)){
                    cash.setAmt14(NumberUtil.add(cash.getAmt14(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }

                //分配股利、利润或偿付利息支付的现金
                if (ArrayUtil.contains(new String[]{"FH","借款利息"}, subject)){
                    cash.setAmt21(NumberUtil.add(cash.getAmt21(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }

                //分配股利、利润或偿付利息支付的现金
                if (ArrayUtil.contains(new String[]{"退投资款"}, subject)){
                    cash.setAmt22(NumberUtil.add(cash.getAmt22(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }

                //收到其他与经营活动有关的现金
                if (ArrayUtil.contains(new String[]{"存款利息"}, subject)){
                    cash.setAmt3(NumberUtil.sub(cash.getAmt3(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
            }
        }

        //上期支出
        List<Map> lastPayDataList = getYdFormDataList(_matchFormUuid("PAY"),JSONObject.toJSONString(UtilMap.map("textField_lqvpypje", lastYear.substring(0,4) + "-" + lastYear.substring(4))),YDConf.FORM_QUERY.retrieve_list);

        if (Objects.nonNull(lastPayDataList) && !lastPayDataList.isEmpty()) {
            for (Map lastPayData : lastPayDataList) {
                Map formData = (Map) lastPayData.get("formData");
                String subject = formData.get("selectField_lp9j90w8") == null ? "" : formData.get("selectField_lp9j90w8").toString();
                //支付给职工以及为职工支付的现金
                if (ArrayUtil.contains(new String[]{"薪酬及附件","社保及公积金"}, subject)) {
                    cash.setLastAmt5(NumberUtil.add(cash.getLastAmt5(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //支付的各项税费
                if (ArrayUtil.contains(new String[]{"经营税费"}, subject)) {
                    cash.setLastAmt6(NumberUtil.add(cash.getLastAmt6(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
                //支付其他与经营活动有关的现金
                if (ArrayUtil.contains(new String[]{"福利费","交通费","差旅费","业务招待费","办公杂费","会议/培训费","快递费","通讯费/网费","广告宣传费","推广佣金","软件/系统服务费","办公设备/软件","物管水电","办公房屋费用","公车费用","代办手续费/税费","客户退款","外包服务费","押金","往来款","手续费"}, subject)){
                    cash.setLastAmt7(NumberUtil.add(cash.getLastAmt7(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }

                //投资支付的现金
                if (ArrayUtil.contains(new String[]{"对外投资款"}, subject)){
                    cash.setLastAmt14(NumberUtil.add(cash.getLastAmt14(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }

                //分配股利、利润或偿付利息支付的现金
                if (ArrayUtil.contains(new String[]{"FH","借款利息"}, subject)){
                    cash.setLastAmt21(NumberUtil.add(cash.getLastAmt21(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }

                //分配股利、利润或偿付利息支付的现金
                if (ArrayUtil.contains(new String[]{"退投资款"}, subject)){
                    cash.setLastAmt22(NumberUtil.add(cash.getLastAmt22(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }

                //收到其他与经营活动有关的现金
                if (ArrayUtil.contains(new String[]{"存款利息"}, subject)){
                    cash.setLastAmt3(NumberUtil.sub(cash.getLastAmt3(), formData.get("numberField_lp82nads").toString()).setScale(2,RoundingMode.HALF_UP).toString());
                }
            }
        }

        List<Cash> cashList = new ArrayList<>();
        cashList.add(cash);

        Map mainData=new HashMap();
        mainData.put("year",year);

        Map map = new HashMap();
        map.put("mainData",mainData);
        map.put("dataList",cashList);

        return map;
    }

    private Map getSubCash(String year) {
        //本期金额
        String year2 = Integer.parseInt(year.substring(0, 4)) + "-" + year.substring(4);

        //去年同期
        String lastYear = (Integer.parseInt(year.substring(0, 4)) - 1) + year.substring(4);
        String lastYear2 = (Integer.parseInt(year2.substring(0, 4)) - 1) + year2.substring(4);

        //本期利润
        List<Map> profitDataList = (List<Map>) ydClient.queryData(YDParam.builder().formUuid(_matchFormUuid("PROFIT"))
                .searchCondition(JSONObject.toJSONString(UtilMap.map("textField_lqvp89br", year2)))
                .build(), YDConf.FORM_QUERY.retrieve_list).getData();
        Cash cash = new Cash();

        if (Objects.nonNull(profitDataList) && !profitDataList.isEmpty()){
            for (Map profitData : profitDataList) {
                Map formData = (Map) profitData.get("formData");
                //净利润
                cash.setAmt1(NumberUtil.add(cash.getAmt1(), formData.get("numberField_lqqhf8yn").toString()).setScale(2,RoundingMode.HALF_UP).toString());
            }
        }


        //上期利润
        List<Map> lastProfitDataList = (List<Map>) ydClient.queryData(YDParam.builder().formUuid(_matchFormUuid("PROFIT"))
                .searchCondition(JSONObject.toJSONString(UtilMap.map("textField_lqvp89br", lastYear2)))
                .build(), YDConf.FORM_QUERY.retrieve_list).getData();
        if (Objects.nonNull(lastProfitDataList) && !lastProfitDataList.isEmpty()){
            for (Map lastProfitData : lastProfitDataList) {
                Map formData = (Map) lastProfitData.get("formData");
                //净利润
                cash.setLastAmt1(NumberUtil.add(cash.getLastAmt1(), formData.get("numberField_lqqhf8yn").toString()).setScale(2,RoundingMode.HALF_UP).toString());
            }
        }


        //获取本月及之前的所有收入
        List<Map> incomeJsonString = new ArrayList<>();
        incomeJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvp25z0",year,"TEXT","le","TextField"));
        incomeJsonString.add(COMPLETED_STATUS);
        incomeJsonString.add(AGREE_RESULT);
        String incomeSearchCondition = JSONObject.toJSONString(incomeJsonString);
        List<Map> incomeList = getYdFormDataList(_matchFormUuid("INCOME"),incomeSearchCondition,YDConf.FORM_QUERY.retrieve_list);

        //获取本月及之前的所有支出
        List<Map> payJsonString = new ArrayList<>();
        payJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvpypje",year2,"TEXT","le","TextField"));
        payJsonString.add(COMPLETED_STATUS);
        payJsonString.add(AGREE_RESULT);
        String paySearchCondition = JSONObject.toJSONString(payJsonString);
        List<Map> payList = getYdFormDataList(_matchFormUuid("PAY"),paySearchCondition,YDConf.FORM_QUERY.retrieve_list);

        //获取资产负债表 本月货币资金
        String initialAmt = "3061971.92";
        String monthAmt = "0.00";
        if (!incomeList.isEmpty()){
            for (Map income : incomeList) {
                Map formData = (Map) income.get("formData");
                if (Objects.nonNull(formData.get("textField_lqvp25z0"))  && Objects.nonNull(formData.get("selectField_lqga7tw0"))){
                    initialAmt = NumberUtil.add(initialAmt,getBigDecimalStringAmt(formData.get("numberField_lp8278sd"))).toString();
                    if (formData.get("textField_lqvp25z0").toString().equals(year)){
                        monthAmt = NumberUtil.add(monthAmt,getBigDecimalStringAmt(formData.get("numberField_lp8278sd"))).toString();
                    }
                }
            }
        }
        if (!payList.isEmpty()){
            for (Map pay : payList) {
                Map formData = (Map) pay.get("formData");
                if (Objects.nonNull(formData.get("textField_lqvpypje"))  && Objects.nonNull(formData.get("multiSelectField_lv0bm8qs"))){
                    List<Map> accounts = (List<Map>) formData.get("multiSelectField_lv0bm8qs");
                    if (!accounts.isEmpty()){
                        initialAmt = NumberUtil.sub(initialAmt,getBigDecimalStringAmt(formData.get("numberField_lp82nads"))).toString();
                        if (formData.get("textField_lqvpypje").toString().equals(year2)){
                            monthAmt = NumberUtil.sub(monthAmt,getBigDecimalStringAmt(formData.get("numberField_lp82nads"))).toString();
                        }
                    }
                }
            }
        }
        cash.setAmt9(initialAmt);

        //获取资产负债表 上月货币资金
        cash.setAmt10(NumberUtil.sub(initialAmt,monthAmt).toString());




        //获取去年同期及之前的所有收入
        List<Map> lastIncomeJsonString = new ArrayList<>();
        lastIncomeJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvp25z0",lastYear,"TEXT","le","TextField"));
        lastIncomeJsonString.add(COMPLETED_STATUS);
        lastIncomeJsonString.add(AGREE_RESULT);
        String lastIncomeSearchCondition = JSONObject.toJSONString(lastIncomeJsonString);
        List<Map> lastIncomeList = getYdFormDataList(_matchFormUuid("INCOME"),lastIncomeSearchCondition,YDConf.FORM_QUERY.retrieve_list);

        //获取去年同期及之前的所有支出
        List<Map> lastPayJsonString = new ArrayList<>();
        lastPayJsonString.add(UtilMap.map("key, value, type, operator, componentName", "textField_lqvpypje",lastYear2,"TEXT","le","TextField"));
        lastPayJsonString.add(COMPLETED_STATUS);
        lastPayJsonString.add(AGREE_RESULT);
        String lastPaySearchCondition = JSONObject.toJSONString(lastPayJsonString);
        List<Map> lastPayList = getYdFormDataList(_matchFormUuid("PAY"),lastPaySearchCondition,YDConf.FORM_QUERY.retrieve_list);

        //获取资产负债表 去年同期货币资金
        String initialAmt2 = "3061971.92";
        String monthAmt2 = "0.00";
        if (!lastIncomeList.isEmpty()){
            for (Map lastIncome : lastIncomeList) {
                Map formData = (Map) lastIncome.get("formData");
                if (Objects.nonNull(formData.get("textField_lqvp25z0"))  && Objects.nonNull(formData.get("selectField_lqga7tw0"))){
                    initialAmt2 = NumberUtil.add(initialAmt2,getBigDecimalStringAmt(formData.get("numberField_lp8278sd"))).toString();
                    if (formData.get("textField_lqvp25z0").toString().equals(lastYear)){
                        monthAmt2 = NumberUtil.add(monthAmt2,getBigDecimalStringAmt(formData.get("numberField_lp8278sd"))).toString();
                    }
                }
            }
        }
        if (!lastPayList.isEmpty()){
            for (Map lastPay : lastPayList) {
                Map formData = (Map) lastPay.get("formData");
                if (Objects.nonNull(formData.get("textField_lqvpypje"))  && Objects.nonNull(formData.get("multiSelectField_lv0bm8qs"))){
                    List<Map> accounts = (List<Map>) formData.get("multiSelectField_lv0bm8qs");
                    if (!accounts.isEmpty()){
                        initialAmt2 = NumberUtil.sub(initialAmt2,getBigDecimalStringAmt(formData.get("numberField_lp82nads"))).toString();
                        if (formData.get("textField_lqvpypje").toString().equals(lastYear2)){
                            monthAmt2 = NumberUtil.sub(monthAmt2,getBigDecimalStringAmt(formData.get("numberField_lp82nads"))).toString();
                        }
                    }
                }
            }
        }

        cash.setLastAmt9(initialAmt2);

        //获取资产负债表 上月货币资金
        cash.setLastAmt10(NumberUtil.sub(initialAmt2,monthAmt2).toString());




        List<Cash> cashList = new ArrayList<>();
        cashList.add(cash);

        Map mainData=new HashMap();
        mainData.put("year",year);

        Map map = new HashMap();
        map.put("mainData",mainData);
        map.put("dataList",cashList);

        return map;

    }

    private List<Accounts> getOtherPayableData(String year, Accounts[] ACCOUNTS_SUM) {
        String dateParam = getYearParam(year);
        // 查询宜搭底表数据
        List<Map> list = getYdFormDataList(_matchFormUuid("OTHER_PAYABLE"),JSONObject.toJSONString(UtilMap.map("dateField_lqyx5k0n", dateParam)),YDConf.FORM_QUERY.retrieve_list_all);

        Map<String,Map<String,Accounts>> dataMap=new HashMap<>();
        // 数据预处理
        for (Map map:list){
            JSONObject formData=JSONObject.parseObject(String.valueOf(map.get("formData")));
            String subject = formData.getString("textField_lqyx5k16") ;// 科目

            if (ArrayUtil.contains(OTHER_PAYABLE_CLASSIFYS,subject)){
                //获取其他应收款明细子表单数据
                List<Map> detailList = (List<Map>) formData.get("tableField_lqyx5k0c");
                Map<String,Accounts> accountsMap = new HashMap<>();
                for (Map detail : detailList) {
                    Accounts accounts = new Accounts();
                    accounts.setCustomerName(detail.get("textField_lqyx5k0f").toString().isEmpty() ? "--" : detail.get("textField_lqyx5k0f").toString());// 客户名称
                    accounts.setInitialAmt(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0o")));// 期初余额
                    accounts.setAmt1(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0q")));// 1月金额
                    accounts.setAmt2(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0r"))); // 2月金额
                    accounts.setAmt3(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0s"))); // 3月金额
                    accounts.setAmt4(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0t"))); // 4月金额
                    accounts.setAmt5(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0u"))); // 5月金额
                    accounts.setAmt6(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0v"))); // 6月金额
                    accounts.setAmt7(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0w"))); // 7月金额
                    accounts.setAmt8(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0x"))); // 8月金额
                    accounts.setAmt9(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0y"))); // 9月金额
                    accounts.setAmt10(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0z"))); // 10月金额
                    accounts.setAmt11(getBigDecimalStringAmt(detail.get("numberField_lqyx5k10"))); // 11月金额
                    accounts.setAmt12(getBigDecimalStringAmt(detail.get("numberField_lqyx5k11"))); // 12月金额
                    //获取原始客户应收账款明细
                    Accounts oldAccounts = accountsMap.get(accounts.getCustomerName());
                    if (Objects.nonNull(oldAccounts)){
                        accounts.add(oldAccounts);
                    }
                    accountsMap.put(accounts.getCustomerName(),accounts);
                }
                Map<String, Accounts> oldAccountsMap = dataMap.get(subject);
                if (Objects.nonNull(oldAccountsMap)){
                    //找出accountsMap和oldAccountsMap相同的key
                    Set<String> customerSet = accountsMap.keySet();
                    for (String oldCustomer : oldAccountsMap.keySet()) {
                        if (customerSet.contains(oldCustomer)){
                            accountsMap.put(oldCustomer,oldAccountsMap.get(oldCustomer).add(accountsMap.get(oldCustomer)));
                        }else {
                            accountsMap.put(oldCustomer,oldAccountsMap.get(oldCustomer));
                        }
                    }
                }
                dataMap.put(subject,accountsMap);
            }
        }

        List<Accounts> accountsList=new ArrayList<>();
        Accounts nullAccounts=new Accounts();
        Accounts totalAccounts = new Accounts();
        // 根据 子科目 处理数据
        for (int i = 0; i < OTHER_PAYABLE_CLASSIFYS.length; i++) {
            List<Accounts> list1=new ArrayList<>();
            Accounts subTotalAccounts=new Accounts();
            if(dataMap.containsKey(OTHER_PAYABLE_CLASSIFYS[i])){
                for (String str:dataMap.get(OTHER_PAYABLE_CLASSIFYS[i]).keySet()){
                    Accounts accountsTemp = dataMap.get(OTHER_PAYABLE_CLASSIFYS[i]).get(str);
                    list1.add(accountsTemp);
                    subTotalAccounts.setInitialAmt(NumberUtil.add(subTotalAccounts.getInitialAmt(),accountsTemp.getInitialAmt()).toString());
                    for (int j = 1; j < 13; j++) {
                        String amtTemp=ReflectUtil.invoke(accountsTemp,"getAmt"+j);
                        String amt=ReflectUtil.invoke(subTotalAccounts,"getAmt"+j);
                        ReflectUtil.invoke(subTotalAccounts, "setAmt"+j, NumberUtil.add(amt,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                    }
                }
            }
            // 补充空数据
            for (int j = list1.size(); j < OTHER_PAYABLE_NUMBERS[i]; j++) {
                list1.add(nullAccounts);
            }
            // 添加小计
            ACCOUNTS_SUM[i]=subTotalAccounts;
            list1.add(ACCOUNTS_SUM[i]);
            totalAccounts.add(subTotalAccounts);
            if (i == OTHER_PAYABLE_NUMBERS.length - 1){
                //添加合计
                list1.add(totalAccounts);
            }
            accountsList.addAll(list1);
        }
        return accountsList;
    }

    private List<Accounts> getOtherReceivableData(String year, Accounts[] ACCOUNTS_SUM) {
        String dateParam = getYearParam(year);

        // 查询宜搭底表数据
        List<Map> list = getYdFormDataList(_matchFormUuid("OTHER_RECEIVABLE"),JSONObject.toJSONString(UtilMap.map("dateField_lqyx5k0n", dateParam)),YDConf.FORM_QUERY.retrieve_list_all);

        Map<String,Map<String,Accounts>> dataMap=new HashMap<>();
        // 数据预处理
        for (Map map:list){
            JSONObject formData=JSONObject.parseObject(String.valueOf(map.get("formData")));
            String subject = formData.getString("textField_lqyx5k16") ;// 科目

            if (ArrayUtil.contains(OTHER_RECEIVABLE_CLASSIFYS,subject)){
                //获取其他应收款明细子表单数据
                List<Map> detailList = (List<Map>) formData.get("tableField_lqyx5k0c");
                Map<String,Accounts> accountsMap = new HashMap<>();
                for (Map detail : detailList) {
                    Accounts accounts = new Accounts();
                    accounts.setCustomerName(detail.get("textField_lqyx5k0f").toString().isEmpty() ? "--" : detail.get("textField_lqyx5k0f").toString());// 客户名称
                    accounts.setInitialAmt(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0o")));// 期初余额
                    accounts.setAmt1(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0q")));// 1月金额
                    accounts.setAmt2(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0r"))); // 2月金额
                    accounts.setAmt3(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0s"))); // 3月金额
                    accounts.setAmt4(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0t"))); // 4月金额
                    accounts.setAmt5(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0u"))); // 5月金额
                    accounts.setAmt6(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0v"))); // 6月金额
                    accounts.setAmt7(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0w"))); // 7月金额
                    accounts.setAmt8(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0x"))); // 8月金额
                    accounts.setAmt9(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0y"))); // 9月金额
                    accounts.setAmt10(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0z"))); // 10月金额
                    accounts.setAmt11(getBigDecimalStringAmt(detail.get("numberField_lqyx5k10"))); // 11月金额
                    accounts.setAmt12(getBigDecimalStringAmt(detail.get("numberField_lqyx5k11"))); // 12月金额
                    //获取原始客户应收账款明细
                    Accounts oldAccounts = accountsMap.get(accounts.getCustomerName());
                    if (Objects.nonNull(oldAccounts)){
                        accounts.add(oldAccounts);
                    }
                    accountsMap.put(accounts.getCustomerName(),accounts);
                }
                Map<String, Accounts> oldAccountsMap = dataMap.get(subject);
                if (Objects.nonNull(oldAccountsMap)){
                    //找出accountsMap和oldAccountsMap相同的key
                    Set<String> customerSet = accountsMap.keySet();
                    for (String oldCustomer : oldAccountsMap.keySet()) {
                        if (customerSet.contains(oldCustomer)){
                            accountsMap.put(oldCustomer,oldAccountsMap.get(oldCustomer).add(accountsMap.get(oldCustomer)));
                        }else {
                            accountsMap.put(oldCustomer,oldAccountsMap.get(oldCustomer));
                        }
                    }
                }
                dataMap.put(subject,accountsMap);
            }
        }

        List<Accounts> accountsList=new ArrayList<>();
        Accounts nullAccounts=new Accounts();
        Accounts totalAccounts = new Accounts();
        // 根据 子科目 处理数据
        for (int i = 0; i < OTHER_RECEIVABLE_CLASSIFYS.length; i++) {
            List<Accounts> list1=new ArrayList<>();
            Accounts subTotalAccounts=new Accounts();
            if(dataMap.containsKey(OTHER_RECEIVABLE_CLASSIFYS[i])){
                for (String str:dataMap.get(OTHER_RECEIVABLE_CLASSIFYS[i]).keySet()){
                    Accounts accountsTemp = dataMap.get(OTHER_RECEIVABLE_CLASSIFYS[i]).get(str);
                    list1.add(accountsTemp);
                    subTotalAccounts.setInitialAmt(NumberUtil.add(subTotalAccounts.getInitialAmt(),accountsTemp.getInitialAmt()).toString());
                    for (int j = 1; j < 13; j++) {
                        String amtTemp=ReflectUtil.invoke(accountsTemp,"getAmt"+j);
                        String amt=ReflectUtil.invoke(subTotalAccounts,"getAmt"+j);
                        ReflectUtil.invoke(subTotalAccounts, "setAmt"+j, NumberUtil.add(amt,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                    }
                }
            }
            // 补充空数据
            for (int j = list1.size(); j < OTHER_RECEIABLE_NUMBERS[i]; j++) {
                list1.add(nullAccounts);
            }
            // 添加小计
            ACCOUNTS_SUM[i]=subTotalAccounts;
            list1.add(ACCOUNTS_SUM[i]);
            totalAccounts.add(subTotalAccounts);
            if (i == OTHER_RECEIABLE_NUMBERS.length - 1){
                //添加合计
                list1.add(totalAccounts);
            }
            accountsList.addAll(list1);
        }
        return accountsList;
    }

    private List<Accounts> getReceivableData(String year, Accounts[] ACCOUNTS_SUM) {
        String dateParam = getYearParam(year);

        // 查询宜搭底表数据
        List<Map> list = getYdFormDataList(_matchFormUuid("RECEIVABLE"),JSONObject.toJSONString(UtilMap.map("dateField_lqyx5k0n", dateParam)),YDConf.FORM_QUERY.retrieve_list_all);

        Map<String,Map<String,Accounts>> dataMap=new HashMap<>();
        // 数据预处理
        for (Map map:list){
            JSONObject formData=JSONObject.parseObject(String.valueOf(map.get("formData")));
            String subject = formData.getString("textField_lqyx5k16") ;// 科目
            if (!"平台服务费".equals(subject) && !"技术服务费".equals(subject)){
                subject = "其他";
            }
            //获取应收账款明细子表单数据
            List<Map> detailList = (List<Map>) formData.get("tableField_lqyx5k0c");
            Map<String,Accounts> accountsMap = new HashMap<>();
            for (Map detail : detailList) {
                Accounts accounts = new Accounts();
                accounts.setCustomerName(detail.get("textField_lqyx5k0f").toString().isEmpty() ? "--" : detail.get("textField_lqyx5k0f").toString());// 客户名称
                accounts.setInitialAmt(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0o")));// 期初余额
                accounts.setAmt1(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0q")));// 1月金额
                accounts.setAmt2(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0r"))); // 2月金额
                accounts.setAmt3(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0s"))); // 3月金额
                accounts.setAmt4(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0t"))); // 4月金额
                accounts.setAmt5(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0u"))); // 5月金额
                accounts.setAmt6(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0v"))); // 6月金额
                accounts.setAmt7(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0w"))); // 7月金额
                accounts.setAmt8(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0x"))); // 8月金额
                accounts.setAmt9(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0y"))); // 9月金额
                accounts.setAmt10(getBigDecimalStringAmt(detail.get("numberField_lqyx5k0z"))); // 10月金额
                accounts.setAmt11(getBigDecimalStringAmt(detail.get("numberField_lqyx5k10"))); // 11月金额
                accounts.setAmt12(getBigDecimalStringAmt(detail.get("numberField_lqyx5k11"))); // 12月金额
                //获取原始客户应收账款明细
                Accounts oldAccounts = accountsMap.get(accounts.getCustomerName());
                if (Objects.nonNull(oldAccounts)){
                    accounts.add(oldAccounts);
                }
                accountsMap.put(accounts.getCustomerName(),accounts);
            }
            Map<String, Accounts> oldAccountsMap = dataMap.get(subject);
            if (Objects.nonNull(oldAccountsMap)){
                //找出accountsMap和oldAccountsMap相同的key
                Set<String> customerSet = accountsMap.keySet();
                for (String oldCustomer : oldAccountsMap.keySet()) {
                    if (customerSet.contains(oldCustomer)){
                        accountsMap.put(oldCustomer,oldAccountsMap.get(oldCustomer).add(accountsMap.get(oldCustomer)));
                    }else {
                        accountsMap.put(oldCustomer,oldAccountsMap.get(oldCustomer));
                    }
                }
            }
            dataMap.put(subject,accountsMap);
        }

        List<Accounts> accountsList=new ArrayList<>();
        Accounts nullAccounts=new Accounts();
        Accounts totalAccounts = new Accounts();
        // 根据 子科目 处理数据
        for (int i = 0; i < RECEIVABLE_CLASSIFYS.length; i++) {
            List<Accounts> list1=new ArrayList<>();
            Accounts subTotalAccounts=new Accounts();
            if(dataMap.containsKey(RECEIVABLE_CLASSIFYS[i])){
                for (String str:dataMap.get(RECEIVABLE_CLASSIFYS[i]).keySet()){
                    Accounts accountsTemp = dataMap.get(RECEIVABLE_CLASSIFYS[i]).get(str);
                    list1.add(accountsTemp);
                    subTotalAccounts.setInitialAmt(NumberUtil.add(subTotalAccounts.getInitialAmt(),accountsTemp.getInitialAmt()).toString());
                    for (int j = 1; j < 13; j++) {
                        String amtTemp=ReflectUtil.invoke(accountsTemp,"getAmt"+j);
                        String amt=ReflectUtil.invoke(subTotalAccounts,"getAmt"+j);
                        ReflectUtil.invoke(subTotalAccounts, "setAmt"+j, NumberUtil.add(amt,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                    }
                }
            }
            // 补充空数据
            for (int j = list1.size(); j < RECEIABLE_NUMBERS[i]; j++) {
                list1.add(nullAccounts);
            }
            // 添加小计
            ACCOUNTS_SUM[i]=subTotalAccounts;
            list1.add(ACCOUNTS_SUM[i]);
            totalAccounts.add(subTotalAccounts);
            if (i > 0){
                //添加合计
                list1.add(totalAccounts);
                totalAccounts = new Accounts();
            }
            accountsList.addAll(list1);
        }
        return accountsList;
    }

    private static String getYearParam(String year) {
        //获取year时间戳区间
        Date yearDate;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            yearDate = dateFormat.parse(year + "-01-01");
        }catch (ParseException e){
            throw new RuntimeException(e);
        }
        long beginDate = DateUtil.beginOfYear(yearDate).getTime();
        long endDate = DateUtil.endOfYear(yearDate).getTime();
        String dateParam = "[" + beginDate + "," + endDate + "]";
        return dateParam;
    }

    private static String getMonthParam(String year) {
        //获取月份时间戳区间
        Date yearDate;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            yearDate = dateFormat.parse(year + "-01");
        }catch (ParseException e){
            throw new RuntimeException(e);
        }
        long beginDate = DateUtil.beginOfMonth(yearDate).getTime();
        long endDate = DateUtil.endOfMonth(yearDate).getTime();
        String dateParam = "[" + beginDate + "," + endDate + "]";
        return dateParam;
    }

    // 按月获取利润表数据
    private List<Profit> getData(String year,Profit[] PROFIT_SUM){
        // 查询宜搭底表数据
        List<Map> list = getYdFormDataList(_matchFormUuid("PROFIT"),JSONObject.toJSONString(UtilMap.map("textField_lqvp89br", year)),YDConf.FORM_QUERY.retrieve_list);


        Map<String,Map<String,Profit>> dataMap=new HashMap<>();
        // 数据预处理（金额数据累加，根据 类型，科目，年月）
        for (Map map:list){
            JSONObject formData=JSONObject.parseObject(String.valueOf(map.get("formData")));
            String classify=formData.getString("textField_lqvp89bt");// 类型
            String subject=formData.getString("textField_lqvp89bu");// 科目
            String amt=formData.getString("numberField_lqqhf8yn");// 金额
            String date=formData.getString("textField_lqvp89br");// 年月 2024-01
            String month=date.split("-")[1].replaceFirst("^0+(?!$)", "");// 月份（去除前缀0）
            // 数据累加
            if(dataMap.containsKey(classify)){
                Map<String,Profit> classifyMap=dataMap.get(classify);
                if(classifyMap.containsKey(subject)){
                    Profit profit=classifyMap.get(subject);
                    // 获取已有数据
                    String amtTemp=ReflectUtil.invoke(profit,"getAmt"+month);
                    ReflectUtil.invoke(profit, "setAmt"+month, NumberUtil.add(amt,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                    dataMap.get(classify).put(subject, profit);
                }else{
                    Profit profit=new Profit(subject);

                    ReflectUtil.invoke(profit, "setAmt"+month, new BigDecimal(amt).setScale(2,RoundingMode.HALF_UP).toString());

                    dataMap.get(classify).put(subject,profit);
                }
            }else{
                Profit profit=new Profit(subject);

                ReflectUtil.invoke(profit, "setAmt"+month, new BigDecimal(amt).setScale(2,RoundingMode.HALF_UP).toString());

                Map<String,Profit> classifyMap=new HashMap<>();
                classifyMap.put(subject,profit);
                dataMap.put(classify,classifyMap);
            }
        }
        List<Profit> profitList=new ArrayList<>();
        Profit nullProfit=new Profit();
        // 根据 类型 处理数据
        for (int i = 0; i < PROFIT_CLASSIFYS.length; i++) {
            List<Profit> list1=new ArrayList<>();
            Profit totalProfit=new Profit(PROFIT_CLASSIFYS[i]+(i==0?"合计":"小计"));
            if(dataMap.containsKey(PROFIT_CLASSIFYS[i])){
                for (String str:dataMap.get(PROFIT_CLASSIFYS[i]).keySet()){
                    list1.add(dataMap.get(PROFIT_CLASSIFYS[i]).get(str).buildTotal());
                    for (int j = 1; j < 13; j++) {
                        String amtTemp=ReflectUtil.invoke(dataMap.get(PROFIT_CLASSIFYS[i]).get(str),"getAmt"+j);
                        String amt=ReflectUtil.invoke(totalProfit,"getAmt"+j);
                        ReflectUtil.invoke(totalProfit, "setAmt"+j, NumberUtil.add(amt,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                    }
                }
            }
            //将list1按照科目降序排序
            Collections.sort(list1,Comparator.comparing(profit -> convertToPinyinFirstChar(profit.getSubject())));
//            list1.sort(Comparator.comparing(Profit::getSubject));
            // 补充空数据
            for (int j = list1.size(); j < PROFIT_NUMBERS[i]; j++) {
                list1.add(nullProfit);
            }

            // 添加小计
            PROFIT_SUM[i]=totalProfit.buildTotal();
            list1.add(PROFIT_SUM[i]);
            profitList.addAll(list1);
        }
        return profitList;
    }

    // 使用Pinyin4j库将汉字转换成拼音首字母
    private static String convertToPinyinFirstChar(String chinese) {
        StringBuilder result = new StringBuilder();
        for (char c : chinese.toCharArray()) {
            String[] pinyinArray = PinyinHelper.toHanyuPinyinStringArray(c);
            if (pinyinArray != null) {
                result.append(pinyinArray[0].charAt(0));
            } else {
                result.append(c);
            }
        }
        return result.toString();
    }

    private List<Profit> getQuarterData(String year,Profit[] PROFIT_SUM){
        // 查询宜搭底表数据
        List<Map> list = getYdFormDataList(_matchFormUuid("PROFIT"),JSONObject.toJSONString(UtilMap.map("textField_lqvp89br", year)),YDConf.FORM_QUERY.retrieve_list);

        Map<String,Map<String,Profit>> dataMap=new HashMap<>();
        // 数据预处理（金额数据累加，根据 类型，科目，年月）
        for (Map map:list){
            JSONObject formData=JSONObject.parseObject(String.valueOf(map.get("formData")));
            String classify=formData.getString("textField_lqvp89bt");// 类型
            String subject=formData.getString("textField_lqvp89bu");// 科目
            String amt=formData.getString("numberField_lqqhf8yn");// 金额
            String date=formData.getString("textField_lqvp89br");// 年月 2024-01
            String month=date.split("-")[1].replaceFirst("^0+(?!$)", "");// 月份（去除前缀0）
            // 数据累加
            if(dataMap.containsKey(classify)){
                Map<String,Profit> classifyMap=dataMap.get(classify);
                if(classifyMap.containsKey(subject)){
                    Profit profit=classifyMap.get(subject);
                    // 获取已有数据
                    String amtTemp=ReflectUtil.invoke(profit,"getAmt"+month);
                    ReflectUtil.invoke(profit, "setAmt"+month, NumberUtil.add(amt,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                    dataMap.get(classify).put(subject, profit);
                }else{
                    Profit profit=new Profit(subject);
                    ReflectUtil.invoke(profit, "setAmt"+month, new BigDecimal(amt).setScale(2,RoundingMode.HALF_UP).toString());
                    dataMap.get(classify).put(subject,profit);
                }
            }else{
                Profit profit=new Profit(subject);
                ReflectUtil.invoke(profit, "setAmt"+month, new BigDecimal(amt).setScale(2,RoundingMode.HALF_UP).toString());
                Map<String,Profit> classifyMap=new HashMap<>();
                classifyMap.put(subject,profit);
                dataMap.put(classify,classifyMap);
            }
        }
        List<Profit> profitList=new ArrayList<>();
        Profit nullProfit=new Profit();
        // 根据 类型 处理数据
        for (int i = 0; i < PROFIT_CLASSIFYS.length; i++) {
            List<Profit> list1=new ArrayList<>();
            Profit totalProfit=new Profit(PROFIT_CLASSIFYS[i]+(i==0?"合计":"小计"));
            if(dataMap.containsKey(PROFIT_CLASSIFYS[i])){
                for (String str:dataMap.get(PROFIT_CLASSIFYS[i]).keySet()){
                    List<Map> budgetList = new ArrayList<>();
                    String dateParam = getYearParam(year);
                    String quarterBudgetAmt = "0.00";
                    int index = 1;
                    if ("收入".equals(PROFIT_CLASSIFYS[i])){
                        // 获取科目季度收入预算金额
                        budgetList = (List<Map>) ydClient.queryData(YDParam.builder().formUuid(_matchFormUuid("INCOME_BUDGET")).searchCondition(
                                JSONObject.toJSONString(UtilMap.map("selectField_lpax809x, selectField_lptilhxv, dateField_lp83xrh9",String.join(", ",str,"季度",dateParam)))
                        ).build(), YDConf.FORM_QUERY.retrieve_list).getData();
                        for (Map map : budgetList) {
                            Map formData = (Map) map.get("formData");
                            String budget = getBigDecimalStringAmt(formData.get("numberField_lp83xrhm"));
                            String quarter = formData.get("selectField_lpuyno7x").toString();
                            quarterBudgetAmt = NumberUtil.add(quarterBudgetAmt, budget).setScale(2,RoundingMode.HALF_UP).toString();
                            if (index == budgetList.size()){
                                switch (quarter){
                                    case "1月-3月":dataMap.get(PROFIT_CLASSIFYS[i]).get(str).setQuarterBudget1(quarterBudgetAmt);break;
                                    case "4月-6月":dataMap.get(PROFIT_CLASSIFYS[i]).get(str).setQuarterBudget2(quarterBudgetAmt);break;
                                    case "7月-9月":dataMap.get(PROFIT_CLASSIFYS[i]).get(str).setQuarterBudget3(quarterBudgetAmt);break;
                                    case "10月-12月":dataMap.get(PROFIT_CLASSIFYS[i]).get(str).setQuarterBudget4(quarterBudgetAmt);break;
                                }
                            }
                            index++;
                        }
                    }else {
                        // 获取科目季度支出预算金额
                        budgetList = (List<Map>) ydClient.queryData(YDParam.builder().formUuid(_matchFormUuid("BUDGET")).searchCondition(
                                JSONObject.toJSONString(UtilMap.map("selectField_lp7ze5jm, selectField_lptilhxv, dateField_lp7ze5kc", String.join(", ",str,"季度",dateParam)))
                        ).build(), YDConf.FORM_QUERY.retrieve_list).getData();
                        for (Map map : budgetList) {
                            Map formData = (Map) map.get("formData");
                            String budget = getBigDecimalStringAmt(formData.get("numberField_lp7ze5ju"));
                            String quarter = formData.get("selectField_lpuyno7x").toString();
                            quarterBudgetAmt = NumberUtil.add(quarterBudgetAmt,budget).setScale(2,RoundingMode.HALF_UP).toString();
                            if (index == budgetList.size()){
                                switch (quarter){
                                    case "1月-3月":dataMap.get(PROFIT_CLASSIFYS[i]).get(str).setQuarterBudget1(quarterBudgetAmt);break;
                                    case "4月-6月":dataMap.get(PROFIT_CLASSIFYS[i]).get(str).setQuarterBudget2(quarterBudgetAmt);break;
                                    case "7月-9月":dataMap.get(PROFIT_CLASSIFYS[i]).get(str).setQuarterBudget3(quarterBudgetAmt);break;
                                    case "10月-12月":dataMap.get(PROFIT_CLASSIFYS[i]).get(str).setQuarterBudget4(quarterBudgetAmt);break;
                                }
                            }
                            index++;
                        }
                    }

                    list1.add(dataMap.get(PROFIT_CLASSIFYS[i]).get(str).buildTotal().buildTotalBudget());
                    for (int j = 1; j < 13; j++) {
                        String amtTemp=ReflectUtil.invoke(dataMap.get(PROFIT_CLASSIFYS[i]).get(str),"getAmt"+j);
                        String amt=ReflectUtil.invoke(totalProfit,"getAmt"+j);
                        //获取季度利润
                        String quarterAmtTemp=ReflectUtil.invoke(dataMap.get(PROFIT_CLASSIFYS[i]).get(str),"getQuarterAmt"+((j + 2) / 3));
                        //月利润累加到季度利润
                        ReflectUtil.invoke(dataMap.get(PROFIT_CLASSIFYS[i]).get(str), "setQuarterAmt"+((j + 2) / 3), NumberUtil.add(quarterAmtTemp,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());

                        ReflectUtil.invoke(totalProfit, "setAmt"+j, NumberUtil.add(amt,amtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                        //一个季度结束时
                        if (j % 3 == 0){
                            //获取原合计/小计季度利润
                            String quarterAmt=ReflectUtil.invoke(totalProfit,"getQuarterAmt"+((j + 2) / 3));
                            //获取当前季度利润
                            quarterAmtTemp=ReflectUtil.invoke(dataMap.get(PROFIT_CLASSIFYS[i]).get(str),"getQuarterAmt"+((j + 2) / 3));
                            //累加设置给合计/小计季度利润
                            ReflectUtil.invoke(totalProfit, "setQuarterAmt"+((j + 2) / 3), NumberUtil.add(quarterAmt,quarterAmtTemp).setScale(2,RoundingMode.HALF_UP).toString());
                            //获取原合计/小计季度预算
                            String quarterBudget = ReflectUtil.invoke(totalProfit, "getQuarterBudget" + ((j + 2) / 3)) == null ? "0" : ReflectUtil.invoke(totalProfit, "getQuarterBudget" + ((j + 2) / 3));
                            //获取当前季度预算
                            String quarterBudgetTemp = ReflectUtil.invoke(dataMap.get(PROFIT_CLASSIFYS[i]).get(str),"getQuarterBudget"+((j + 2) / 3)) == null ? "0" : ReflectUtil.invoke(dataMap.get(PROFIT_CLASSIFYS[i]).get(str),"getQuarterBudget"+((j + 2) / 3));

                            //累加设置给合计/小计季度预算
                            ReflectUtil.invoke(totalProfit, "setQuarterBudget"+((j + 2) / 3), NumberUtil.add(quarterBudget,quarterBudgetTemp).setScale(2,RoundingMode.HALF_UP).toString());

                        }

                    }
                }
            }
            // 补充空数据
            for (int j = list1.size(); j < PROFIT_NUMBERS[i]; j++) {
                list1.add(nullProfit);
            }
            // 添加小计
            PROFIT_SUM[i]=totalProfit.buildTotal().buildTotalBudget();
            list1.add(PROFIT_SUM[i]);
            profitList.addAll(list1);
        }
        return profitList;
    }

    private String addAmt(Profit[] PROFIT_SUM,int start,int end,int i){
        String result="0";
        for (int j = start; j <= end; j++) {
            result = NumberUtil.add(result,ReflectUtil.invoke(PROFIT_SUM[j],"getAmt"+i)).toString();
        }
        return result;
    }

    private String subAmt(Profit[] PROFIT_SUM,int start,int end,int i){
        String result=ReflectUtil.invoke(PROFIT_SUM[0],"getAmt"+i);// 0 是收入，其他是支出;
        if(StrUtil.isBlankIfStr(result)){
            result = "0";
        }
        for (int j = start; j <= end; j++) {
            result = NumberUtil.sub(result,ReflectUtil.invoke(PROFIT_SUM[j],"getAmt"+i)).toString();
        }
        return result;
    }

    private String addQuarterAmt(Profit[] PROFIT_SUM,int start,int end,int i){
        String result="0";
        for (int j = start; j <= end; j++) {
            result = NumberUtil.add(result,ReflectUtil.invoke(PROFIT_SUM[j],"getQuarterAmt"+i)).toString();
        }
        return result;
    }

    private String subQuarterAmt(Profit[] PROFIT_SUM,int start,int end,int i){
        String result=ReflectUtil.invoke(PROFIT_SUM[0],"getQuarterAmt"+i);// 0 是收入，其他是支出;
        if(StrUtil.isBlankIfStr(result)){
            result = "0";
        }
        for (int j = start; j <= end; j++) {
            result = NumberUtil.sub(result,ReflectUtil.invoke(PROFIT_SUM[j],"getQuarterAmt"+i)).toString();
        }
        return result;
    }

    private String addBudgetQuarterAmt(Profit[] PROFIT_SUM,int start,int end,int i){
        String result="0";
        for (int j = start; j <= end; j++) {
            result = NumberUtil.add(result,ReflectUtil.invoke(PROFIT_SUM[j],"getQuarterBudget"+i)).toString();
        }
        return result;
    }

    private String subBudgetQuarterAmt(Profit[] PROFIT_SUM,int start,int end,int i){
        String result=ReflectUtil.invoke(PROFIT_SUM[0],"getQuarterBudget"+i);// 0 是收入，其他是支出;
        if(StrUtil.isBlankIfStr(result)){
            result = "0";
        }
        for (int j = start; j <= end; j++) {
            result = NumberUtil.sub(result,ReflectUtil.invoke(PROFIT_SUM[j],"getQuarterBudget"+i)).toString();
        }
        return result;
    }

}
